# configuration -- all the shell files pick this up using "."
# this makes it possible to add new variables here and have them
#  available everywhere immediately
#
# This is not, repeat NOT, a master control file for all of C News.
# This is the shell equivalent of libcnews/config.c, a "subroutine
# library" that gives shell files access to the default settings and
# lets environment variables override the defaults.  Changing the
# defaults here will *NOT* change them throughout C News.
#
# =()<NEWSCTL=${NEWSCTL-@<NEWSCTL>@}>()=
NEWSCTL=${NEWSCTL-/usr/lib/news}
# =()<NEWSBIN=${NEWSBIN-@<NEWSBIN>@}>()=
NEWSBIN=${NEWSBIN-/usr/lib/newsbin}
# =()<NEWSARTS=${NEWSARTS-@<NEWSARTS>@}>()=
NEWSARTS=${NEWSARTS-/usr/spool/news}
# =()<NEWSPATH=${NEWSPATH-@<NEWSPATH>@}>()=
NEWSPATH=${NEWSPATH-/bin:/usr/bin}
# =()<NEWSUMASK=${NEWSUMASK-@<NEWSUMASK>@}>()=
NEWSUMASK=${NEWSUMASK-002}
# =()<NEWSMASTER=${NEWSMASTER-@<NEWSMASTER>@}>()=
NEWSMASTER=${NEWSMASTER-usenet}
# =()<NEWSCONFIG=${NEWSCONFIG-@<NEWSCONFIG>@}>()=
NEWSCONFIG=${NEWSCONFIG-/usr/lib/news/bin/config}
